/*

=head1 NAME

Upgrade - Upgrade older JavaScript implementations to support newer features

=head1 SYNOPSIS

  // Upgrade Array.push in a client-side JSAN.js context
  var jsan = new JSAN('/js');
  jsan.use('Upgrade.Array.push');

=head1 DESCRIPTION

Many many different JavaScript toolkits start with something like the following:

  // Provides Array.push for implementations that don't have it
  if ( ! Array.prototype.push ) {
      Array.prototype.push = function () {
          var l = this.length;
          for ( var i = 0; i < arguments.length; i++ ) {
              this[l+i] = arguments[i];
          }
          return this.length;
      }
  }

These provide implementations of expected or required functions/classes
for older JavaScript implementations that do not provide them natively,
in effect "upgrading" the client at run-time.

In fact, due to it's flexibility JavaScript is a language ideally suited
to this sort of behaviour.

C<Upgrade> is a JSAN package that provides standard implementations for
many of these standard functions. If your code relied on a particular
function that you later find to be not as common as you might have
initially thought, you can simply add a dependency on that function within
the C<Upgrade> namespace, and the standard code to make sure that
function is available will be added for you.

Rather than one large file that upgrades everything all at once, C<Upgrade>
will be broken down into a large number of smaller .js files, each
implementing one function or class.

=head1 USING UPGRADE

The C<Upgrade> namespace acts as a sort of parallel root to the global
namespace. For any function you want to upgrade, you can then simply prepend
the upgrade namespace to it.

For example, to do the very common upgrade for the C<Array.push> function,
you simply use C<Upgrade.Array.push> (or manually load the
C<Upgrade/Array/push.js> file).

An alternative, available later, will be to include it via a server-side
C<JSAN.Use> server (not yet available). More information on this will be
made available once the server is completed.

=head1 UPGRADABLE FUNCTIONS

Rather than separately document each upgradable function in their own .js
files, we will instead document each of them here, provided in alphabetical
order.

=head2 Upgrade.Array.push

The provides the same standard implementation of the instance method
C<Array.push> (located at Array.prototype.push) as used by all of the
major frameworks.

=head2 Upgrade.Function.apply

This provides a version of the instance method C<Function.apply>
(located at C<Function.prototype.apply>) adapted from the version found
in the L<Prototype> framework, which was itself adapted from an
implementation found on L<http://www.youngpup.net/>.

=head1 METHODS

The C<Upgrade> module itself does not at this time provide any actual
implementation code, and only acts as a source of documentation.

Likewise, nothing is ever actually created underneath the C<Upgrade>
namespace, but serves as an address mechanism for determining which
.js files to load. Each of these inserts functions into the core tree.

=head1 SUPPORT

Non-commerical support is unavailable at this time, until the
bug-tracking system for JSAN is completed.

Contact the author or Phase N (L<www.phase-n.com>) for commercial support.

For B<non-support> issues or questions or to contribute new functions to
be added to Upgrade, contact the author.

=head1 AUTHOR

Adam Kennedy <jsan@ali.as>, L<http://ali.as/>

=head1 COPYRIGHT

Copyright (c) 2005 Adam Kennedy. All rights reserved.
This program is free software; you can redistribute it and/or modify
it under the the terms of the Perl dual GPL/Artistic license.

The full text of the license can be found in the
LICENSE file included with this package

*/
